@@ -1,24 +1,13 @@
 # library(evd)
 
-PLIquantile_multivar = function(order,x,y,inputs,deltasvector,InputDistributions,samedelta=TRUE,percentage=FALSE){
+PLIquantile_multivar = function(order,x,y,inputs,deltasvector,InputDistributions,samedelta=TRUE,percentage=TRUE,nboot=0,conf=0.95,bootsample=TRUE){
   
   # This function allows the estimation of Density Modification Based Reliability Sensitivity Indices
-  # called PLI (Perturbed-Law based sensitivity Indices) for a quantile,
+  # called PLI (Perturbed-Law based sensitivity Indices) for a quantile
   #
   # It is adapted for taking into account simultaneous perturbations of 2 inputs
   #
   # It only works for MEAN perturbation of the inputs
-  
-  # Author: Paul Lemaitre, Bertrand Iooss, Thibault Delage and Roman Sueur
-  #
-  # Refs : P. Lemaitre, E. Sergienko, A. Arnaud, N. Bousquet, F. Gamboa and B. Iooss.
-  #        Density modification based reliability sensitivity analysis, 
-  #        Journal of Statistical Computation and Simulation, 85:1200-1223, 2015.
-  #        hal.archives-ouvertes.fr/docs/00/73/79/78/PDF/Article_v68.pdf.
-  #
-  #        R. Sueur, B. Iooss and T. Delage. 
-  #        Sensitivity analysis using perturbed-law based indices for quantiles and application to an industrial case}, 
-  #        10th International Conference on Mathematical Methods in Reliability (MMR 2017), Grenoble, France, July 2017.
   #
   ###################################
   ## Description of input parameters
@@ -35,6 +24,9 @@ PLIquantile_multivar = function(order,x,y,inputs,deltasvector,InputDistributions
   #   If not, the mean perturbation will be new_mean = mean+sigma*delta where mean, sigma are parameters defined in InputDistributions and delta is a value of deltasvector. See subsection 4.3.3 of the reference for an exemple of use. 
   # percentage defines the formula used for the PLI. If percentage=FALSE, the classical formula used in the bibliographic references is used.
   #   If percentage=TRUE, the PLI is given in percentage of variation of the quantile (even if it is negative). 
+  # nboot is the number of bootstrap replicates
+  # conf is the required bootstrap confidence interval
+  # bootsample defines if the sampling uncertainty is taken into account in computing the boostrap confidence intervals of the PLI
   #
   ###################################
   ## Description of output parameters
@@ -51,7 +43,7 @@ PLIquantile_multivar = function(order,x,y,inputs,deltasvector,InputDistributions
   nmbrededeltas=length(deltasvector)	# number of perturbations
   
   ## some storage matrices 
-  I <- J <- matrix(0,ncol=nmbrededeltas,nrow=nmbrededeltas) 
+  I <- J <- ICIinf <- ICIsup <- JCIinf <- JCIsup <- matrix(0,ncol=nmbrededeltas,nrow=nmbrededeltas) 
 
   ########################################
   ## Definition of useful functions used further on 
@@ -83,6 +75,13 @@ PLIquantile_multivar = function(order,x,y,inputs,deltasvector,InputDistributions
     return(s)
   }
   
+  transinverse <- function(a,b,c) {
+    # useful to compute bootstrap-based confidence intervals
+    if (a > c) ans <- a / b - 1
+    else ans <- 1 - b / a
+    return(ans)
+  }
+  
   ########################################
   ## Principal loop of the function 
   ########################################
@@ -97,6 +96,11 @@ PLIquantile_multivar = function(order,x,y,inputs,deltasvector,InputDistributions
   # First input
   i = inputs[1]
   
+  if (nboot > 0){
+    lqidb=array(0,dim=c(nmbrededeltas,nmbrededeltas,nboot)) # pour bootstrap 
+    quantilehatb=NULL
+  }
+  
   ## definition of local variables
     Loi.Entree=InputDistributions[[i]]
   
@@ -420,6 +424,45 @@ PLIquantile_multivar = function(order,x,y,inputs,deltasvector,InputDistributions
       }
     }
     
+    ###############################################################
+    ##########              BOOTSTRAP                ###############
+    ###############################################################
+    
+    if (nboot >0){
+      for (b in 1:nboot){
+        ib <- sample(1:length(y),replace=TRUE)
+        xb <- x[ib,]
+        yb <- y[ib]
+        
+        quantilehatb <- c(quantilehatb, quantile(yb,order)) # quantile estimate
+        ysb = sort(yb,index.return=T) # ordered output
+        xsb = xb[ysb$ix,] # inputs ordered by increasing output
+        
+        for (K1 in 1:nmbrededeltas){
+          for (K2 in 1:nmbrededeltas){
+            if ((vdd1[K1]!=0) & (vdd2[K2]!=0)){
+              res=NULL		
+              pti1=phi1(vlambda1[K1])
+              pti2=phi2(vlambda2[K2])
+              for (j in 1:nmbredepoints){	
+                res[j]=exp(vlambda1[K1]*xsb[j,inputs[1]]-pti1 + vlambda2[K2]*xsb[j,inputs[2]]-pti2)
+              }
+              sum_res = sum(res)
+              kid = 1
+              res1 = res[1]
+              res2 = res1/sum_res
+              while (res2 < order){
+                kid = kid + 1
+                res1 = res1 + res[kid]
+                res2 = res1/sum_res
+              }
+              lqidb[K1,K2,b] = ysb$x[kid]
+              } else lqidb[K1,K2,b] = quantilehatb[b]
+          }
+        }
+      } # end of bootstrap loop
+    }
+    
   ######################################################################################
   
   ########################################
@@ -428,22 +471,41 @@ PLIquantile_multivar = function(order,x,y,inputs,deltasvector,InputDistributions
     
   for (i in 1:nmbrededeltas){
     for (j in 1:nmbrededeltas){
-      if (percentage==FALSE){
-        if(lqid[i,j]>quantilehat){
-          I[i,j]=lqid[i,j]/quantilehat-1
-          J[i,j]=lqid[i,j]
+      J[i,j]=lqid[i,j]
+      if (percentage==FALSE) I[i,j]=transinverse(lqid[i,j],quantilehat,quantilehat)
+      else I[i,j]=lqid[i,j]/quantilehat-1
+      
+      if (nboot > 0){
+        # ICI = PLI bootstrap including or excluding sampling uncertainty
+        # JCI = quantile bootstrap 
+        qinf <- quantile(lqidb[i,j,],(1-conf)/2)
+        qsup <- quantile(lqidb[i,j,],(1+conf)/2)
+        JCIinf[i,j]=qinf
+        JCIsup[i,j]=qsup
+        qb <- mean(quantilehatb)
+        if (percentage==FALSE){
+          if (bootsample){
+            ICIinf[i,j]=transinverse(qinf,qb,qb)
+            ICIsup[i,j]=transinverse(qsup,qb,quantilehat)
+          } else{
+            ICIinf[i,j]=quantile(transinverse(lqidb[i,j,],quantilehatb,quantilehatb),(1-conf)/2)
+            ICIsup[i,j]=quantile(transinverse(lqidb[i,j,],quantilehatb,quantilehatb),(1+conf)/2)
+          }
         } else {
-          I[i,j]=-quantilehat/lqid[i,j]+1
-          J[i,j]=lqid[i,j]
+          if (bootsample){
+            ICIinf[i,j]=qinf/qb-1
+            ICIsup[i,j]=qsup/qb-1
+          } else{
+            ICIinf[i,j]=quantile((lqidb[i,j,]/quantilehatb-1),(1-conf)/2)
+            ICIsup[i,j]=quantile((lqidb[i,j,]/quantilehatb-1),(1+conf)/2)
+          }
         }
-      } else{
-        I[i,j]=lqid[i,j]/quantilehat-1
-        J[i,j]=lqid[i,j]
       }
     }
   }
-    
-  res <- list(PLI = I, quantile = J)
+
+  res <- list(PLI = I, PLICIinf = ICIinf, PLICIsup = ICIsup, quantile = J, quantileCIinf = JCIinf, quantileCIsup = JCIsup)
+  
   
   return(res)
 }
